@@ -43,7 +43,7 @@ class AuxCodonDicts():
 
 
 
-        human_avtRNA_gene_number_anticodon_IBEN2015 = {
+        self.IBEN2015_tRNA_GCN_averages_by_anticodon = {
             'AGC': 36.23, 'CGC':4.68,
             'UGC':10.07, 'ACG': 6.87,
             'CCG':3.71, 'CCU': 5.48,
@@ -72,10 +72,10 @@ class AuxCodonDicts():
             'UAC':5.46}
 
 
-        tmpdict = self.__convert_codon_to_anticodon(human_avtRNA_gene_number_anticodon_IBEN2015)
-        self.IBEN2015_tRNA_GCN_averages = self.__add_other_keys(tmpdict)
+        #tmpdict = self.__convert_codon_to_anticodon(human_avtRNA_gene_number_anticodon_IBEN2015)
+        #self.IBEN2015_tRNA_GCN_averages = self.__add_other_keys(tmpdict)
 
-        human_stdtRNA_gene_number_anticodon_IBEN2015 = {
+        self.IBEN2015_tRNA_GCN_stds_by_anticodon = {
             'AGC': 2.68, 'CGC':0.65,
             'UGC':1.02, 'ACG': .66,
             'CCG':.4, 'CCU': .59,
@@ -103,11 +103,11 @@ class AuxCodonDicts():
             'AAC':.91, 'CAC':1.78,
             'UAC':.68}
 
-        tmpdict = self.__convert_codon_to_anticodon(human_stdtRNA_gene_number_anticodon_IBEN2015)
-        self.IBEN2015_tRNA_GCN_stds = self.__add_other_keys(tmpdict)
+        #tmpdict = self.__convert_codon_to_anticodon(human_stdtRNA_gene_number_anticodon_IBEN2015)
+        #self.IBEN2015_tRNA_GCN_stds = self.__add_other_keys(tmpdict)
 
 
-        hg19_trna_gene_number_anticodon = {
+        self.hg19_tRNA_GCN_by_anticodon = {
             'AGC': 33, 'CGC':5,
             'UGC':11, 'ACG': 8, 'GGC':2,
             'CCG':4, 'CCU': 8,
@@ -135,8 +135,8 @@ class AuxCodonDicts():
             'AAC':12, 'CAC':18, #duplicate for doubled up codons
             'UAC':7,}
 
-        tmpdict = self.__convert_codon_to_anticodon(hg19_trna_gene_number_anticodon)
-        self.hg19_tRNA_GCN = self.__add_other_keys(tmpdict)
+        #tmpdict = self.__convert_codon_to_anticodon(self.hg19_tRNA_GCN_anticodon)
+        #self.hg19_tRNA_GCN_codon = self.__add_other_keys(tmpdict)
 
 
 
@@ -231,7 +231,7 @@ class AuxCodonDicts():
 
             for i in range(3):
                 if newkeys[i] == 'A':
-                    newkeys[i] = 'T'
+                    newkeys[i] = 'U'
 
                 elif newkeys[i] == 'U':
                     newkeys[i] = 'A'
