@@ -555,7 +555,196 @@ class TranslationSolvers():
 
 
 
-    def solve_ssa(self, ke, t, ki=.33, kt = 10,
+
+    def solve_ssa_protease(self, ke, t, ki=.033, kt = 10, 
+                  x0=[], n_traj=100, bins=None, protease_locations = [], protease_rates = [],
+                  perturb=[0,0,0,0], leaky_probes=False, kprobe=np.ones(1),
+                  record_stats=False, probe_vec=None, probe_loc=None,
+                  kon=1, koff=1, bursting=False, rib_prealloc=200,
+                  dynamic_prealloc=False, photobleaching=False, photobleaching_rate=0.0):
+        '''
+
+        Solve a stochastic simulation of ribosomes on an mRNA transcript
+        given codon dependent rates (TASEP) with protease trimming
+
+        Parameters
+        ----------
+        ke : list
+            list of kelongation rates, can be accessed via a protein object by POI.kelong.
+        t : list, arr
+            a list/array of times to run the simulation for.
+        ki : float, optional
+            Initiation Rate. The default is .033 1/time.
+        kt : float, optional
+            Termination Rate. The default is 10 1/time.
+        x0 : numpy array, optional
+            1xRibsomes max intial condition. The default is []
+            (no intial condition).
+        n_traj : int, optional
+            Number of trajectories to simulate. The default is 100.
+        bins : numpy array, optional
+            binning key array, for example [2,2,2,2,2] would bin a 10 length
+            ke into 5 bins. The default is None.
+        protease_locations : 1D array like, optional
+            list of locations of where protease can trim nascent chains
+        protease_rates : 1D array like, optional
+            list of rates for each protease to trim available sites.
+        low_memory : Bool, optional
+            Use lower memory when simulating, meaning dont keep track of
+            ribosome positions over time, just the Intensity. The default is True.
+        perturb : list [bool, bool, float], optional
+            Apply a pertubation [frap, inhibitor, application time], for
+            example [1,0,50] would apply frap at time 50. The default is [0,0,0].
+        leaky_probes : bool, optional
+            Use leaky probes with a probability kprobe for binding when
+            passing each epitope. The default is False.
+        kprobe : list of floats, optional
+            probabilities for each probe to bind, for example a two color
+            system should have [.1,.9] for 10% chance of color 1 binding,
+            and 90% of color two binding on each epitope.
+            The default is np.ones(1).
+        record_stats : bool, optional
+            Record stats such as collisions or ribosome dwell times.
+            The default is False.
+        probe_vec : numpy array, optional
+            probe cumsum to convert to Intensities Ncolor x L_transcript.
+            If left blank will use POI. The default is None.
+        probe_loc : numpy array, optional
+            probe locations Ncolor x L_transcript with 1 for epitope
+            locations, 0 otherwise. If left blank will use POI.
+            The default is None.
+        kon : float, optional
+            on rate for bursting. The default is 1.
+        koff : float, optional
+            off rate for bursting. The default is 1.
+        bursting : bool, optional
+            use a on/off bursting dynamic for initiation. The default is False.
+        rib_prealloc : int, optional
+            how many ribosomes to consider binding at a time, for a
+            transcript the maximum that can bind at a time is L/9.
+            The default is 200.
+        dynamic_prealloc : bool, optional
+            calculate the expeted number of maximum ribosomes and use that
+            to preallocate memory. The default is False.
+
+        Returns
+        -------
+        ssa_obj : SSA_Soln_obj
+            returns a container object containing all the stats from
+            the simulation, such as intensity.
+
+        '''
+
+
+        self.__check_rates(ke)
+
+        ssa_conditions = self.default_conditions
+        if np.sum(kprobe != 1) !=0:
+            ssa_conditions['leaky_probes'] = True
+            leaky_probes = True
+
+        if isinstance(protease_locations, list):
+            protease_locations = np.array(protease_locations,dtype = np.int32)
+            
+        if isinstance(protease_rates, list):
+            protease_rates = np.array(protease_rates,dtype = np.float64)
+
+        ssa_conditions['perturb'] = perturb
+        ssa_conditions['leaky_probes'] = leaky_probes
+        ssa_conditions['low_mem'] = False
+        ssa_conditions['record_stats'] = record_stats
+        ssa_conditions['bins'] = bins
+        ssa_conditions['bursting'] = bursting
+        ssa_conditions['ribosome_preallocation'] = rib_prealloc
+        ssa_conditions['dynamic_rib_prealloc'] = dynamic_prealloc
+        ssa_conditions['photobleaching'] = photobleaching
+        ssa_conditions['photobleaching_rate'] = photobleaching_rate
+        ssa_conditions['protease_rates'] = protease_rates
+        ssa_conditions['protease_locations'] = protease_locations
+        provided_probe = False
+
+        try:
+            probe_vec[0]
+            provided_probe = True
+        except:
+            pass
+
+        provided_protein = False
+        try:
+            self.protein.kelong[0]
+            provided_protein = True
+
+        except:
+            pass
+
+        if provided_protein: #parse out stop codons if they are included
+            includes_stop = False
+            if self.protein.aa_seq[-1] == '*' or self.protein.nt_seq[-1].upper() in ['UAA','TAA','TAG','UAG', 'UGA','TGA']:
+                if len(self.protein.aa_seq) == len(ke):
+                    includes_stop = True
+
+
+
+        if not provided_probe:
+            if provided_protein:
+                if includes_stop:
+                    probe_vec = np.require(self.protein.probe_vec.astype(np.int32)[:,:-1],requirements=['C'])
+                    probe_loc = np.require(self.protein.probe_loc.astype(np.int32)[:,:-1],requirements=['C'])
+                    ke = ke[:-1]
+                else:
+                    probe_vec = self.protein.probe_vec.astype(np.int32)
+                    probe_loc = self.protein.probe_loc.astype(np.int32)
+            else:
+                probe_loc = np.zeros([1,len(ke)], dtype=np.int32)
+                probe_vec = np.zeros([1,len(ke)], dtype=np.int32)
+                warnings.warn('no provided probe vector, using a blank probe'\
+                              '. This will result in zero sum intensity.')
+                
+        else:
+            probe_vec = probe_vec
+
+
+        ssa_conditions['probe_vec'] = probe_vec
+        ssa_conditions['probe_loc'] = probe_loc
+
+
+        if dynamic_prealloc:
+
+            ssa_conditions['ribosome_preallocation'] = int(len(ke)/ssa_conditions['footprint'])+5
+
+        k = [ki,] + ke + [kt,]
+        if self.cython_available:
+
+            ssa_obj = self.__solve_ssa_replicon(k,t,x0,n_traj,protease_locations, protease_rates, ssa_conditions = ssa_conditions, kon=kon, koff=koff, kprobe=kprobe)
+
+
+        else:
+            warnings.warn('Warning: Protease model is only included in the c++'\
+              'This will return None until you install the translation c++.')
+
+
+       #Generate metadata
+        colors = ssa_obj.intensity_vec.shape[0]
+        st = ssa_obj.start_time
+        ft = ssa_obj.time_rec[-1]
+        lt = len(ssa_obj.time_rec)
+        if bins == None:
+            bstr = 0
+        else:
+            bstr=1
+        sid = 'L' + str(len(k)) + 'N' + str(n_traj) + 'T' + str(st) + '_' + str(ft) + '_' + str(lt)
+        fstr = '-F' + str(int(self.cython_available)) + str(int(sum(perturb))) + str(int(leaky_probes)) + str(int(False))  + str(int(record_stats))  +  str(int(bstr)) + str(int(bursting))
+        cstr = 'C' + str(int(colors))
+        nprobe = np.sum(probe_loc,axis=1)
+        for value in nprobe:
+            cstr = cstr+ 'P' + str(int(value))
+        sid = sid + cstr + fstr
+        ssa_obj._SSA_Soln__meta['id'] = sid
+
+        return ssa_obj
+
+
+    def solve_ssa(self, ke, t, ki=.033, kt = 10,
                   x0=[], n_traj=100, bins=None, low_memory=True,
                   perturb=[0,0,0,0], leaky_probes=False, kprobe=np.ones(1),
                   record_stats=False, probe_vec=None, probe_loc=None,
@@ -573,7 +762,7 @@ class TranslationSolvers():
         t : list, arr
             a list/array of times to run the simulation for.
         ki : float, optional
-            Initiation Rate. The default is .33 1/time.
+            Initiation Rate. The default is .033 1/time.
         kt : float, optional
             Termination Rate. The default is 10 1/time.
         x0 : numpy array, optional
@@ -1577,6 +1766,215 @@ class TranslationSolvers():
         return ssa_obj
 
 
+    def __solve_ssa_replicon(self,k, t, x0, n_traj, protease_locations, protease_rates, ssa_conditions=None,
+                                  kon=1, koff=1, kprobe=[] ):
+        seeds = np.random.randint(0, 0x7FFFFFF, n_traj, dtype=np.int32)
+
+        if isinstance(k, list):
+            k = np.array(k).astype(np.float64)
+
+        k = k.flatten()
+
+        if kprobe == []:
+            kprobe = np.ones(self.color)
+
+        if isinstance(kprobe, list):
+            kprobe = np.array(kprobe, dtype=np.float64)
+        if isinstance(kprobe, int):
+            kprobe = np.array([kprobe], dtype=np.float64)
+        if isinstance(kprobe, float):
+            kprobe = np.array([kprobe], dtype=np.float64)
+
+
+        if ssa_conditions == None:
+            ssa_conditions = self.default_conditions
+
+        x0 = self.__check_x0(x0)
+
+        probe_vec = ssa_conditions['probe_vec']
+
+        colors = self.colors
+
+
+        if ssa_conditions['bursting'] == False:
+            kon = 1
+            koff = 1
+        else:
+            kon = kon
+            koff = koff
+
+        flags = np.array([int(ssa_conditions['bursting']),
+                          int(ssa_conditions['leaky_probes']),
+                          int(ssa_conditions['record_stats'])], dtype=np.int32)
+        probe_loc = ssa_conditions['probe_loc']
+
+        colors = int(probe_loc.shape[0])
+        rib_vec = []
+        solutions = []
+        solutionssave = []
+
+
+        N_rib = ssa_conditions['ribosome_preallocation']
+        all_results,all_nribs,all_collisions,_,all_ribtimes,all_col_points = self.__generate_mats_lowmem(n_traj,k[0],t,N_rib,colors)
+        all_rib_loc = np.zeros((n_traj, len(t), N_rib), dtype=np.int32)
+
+
+        footprint = ssa_conditions['footprint']
+        evf = ssa_conditions['perturb'][0]
+        evi = ssa_conditions['perturb'][1]
+        intime = ssa_conditions['perturb'][2]
+        intime_stop = ssa_conditions['perturb'][3]
+        non_consider_time = ssa_conditions['burnin']
+        photobleaching_rate = ssa_conditions['photobleaching_rate']
+        photobleaching = ssa_conditions['photobleaching']
+        
+         
+        
+        st = time.time()
+
+        for i in range(n_traj):
+
+            nribs = np.array([0], dtype=np.int32)
+            result,ribtimes,_,coltimes,colpointsx,colpointst = self.__generate_vecs_lowmem(k,t,N_rib,colors)
+            ribloc = np.zeros((N_rib,len(t)), dtype=np.int32)
+            if i == 0: #detect any int64
+                inputs = [ribloc,result, ribtimes, coltimes, colpointsx,colpointst, k[1:-1], t, k[0], float(k[-1]), int(evf), int(evi), float(intime), float(intime_stop), seeds[i],nribs,x0,footprint, probe_vec ,int(colors), kon, koff, kprobe, probe_loc, flags,N_rib, photobleaching_rate, photobleaching, protease_locations, protease_locations]
+                wash_inputs = self.__check_input_memview(inputs)
+
+            if wash_inputs:
+                #check memview so all given variables are in int32 if integer for C
+                inputs = [ribloc,result, ribtimes, coltimes, colpointsx,colpointst, k[1:-1], t, k[0], float(k[-1]), int(evf), int(evi), float(intime), float(intime_stop), seeds[i],nribs,x0,footprint, probe_vec ,int(colors), kon, koff, kprobe, probe_loc, flags,N_rib, photobleaching_rate, photobleaching, protease_locations, protease_locations]
+                ribloc,result, ribtimes, coltimes, colpointsx,colpointst, kelong, t, ki, kt, evf, evi, intime,intime_stop, seed,nribs,x0,footprint, probe_vec ,colors, kon, koff, kprobe, probe_loc, flags, N_rib, photobleaching_rate, photobleaching, protease_locations, protease_locations = self.__check_memview(inputs)
+            else:
+                kelong = k[1:-1]
+                ki = k[0]
+                kt = float(k[-1])
+                evf = int(evf)
+                evi = int(evi)
+                intime = float(intime)
+                intime_stop = float(intime_stop)
+                seed = seeds[i]
+                colors = int(colors)
+
+            
+            ssa_translation_lowmem.run_SSA_replicon(ribloc, result, ribtimes, coltimes, colpointsx,colpointst, kelong, t, ki, kt, evf, evi, intime, intime_stop, seed,nribs,x0,footprint, probe_vec ,colors, kon, koff, kprobe, probe_loc, flags, photobleaching_rate, photobleaching,protease_locations, protease_rates)
+            #ssa_translation.run_SSA(result, ribtimes, coltimes, k[1:-1],frapresult, truetime, k[0], k[-1], evf, evi, intime, seeds[i],nribs)
+
+            all_results[i, :, :] = result.T
+            #all_frapresults[i, :] = frapresult
+            all_ribtimes[i, :] = ribtimes
+            all_collisions[i, :] = coltimes
+            all_nribs[i, :] = nribs
+            all_rib_loc[i, :, :] = ribloc.T
+
+            endcolrec = np.where(colpointsx == 0)[0][0]
+
+            colpoints = np.vstack((colpointsx[:endcolrec],
+                                   colpointst[:endcolrec]))
+            all_col_points.append(colpoints.T)
+
+
+
+            # for i in range(n_traj):
+            #     soln = all_results[i,:, :]
+
+            #     so = soln
+            #     solutionssave.append(so)
+            #     solutions.append(soln)
+
+            collisions = np.array([[]])
+            watched_ribs = []
+            for i in range(n_traj):
+                totalrib = all_nribs[i]
+
+                if totalrib > all_collisions.shape[1]:
+                    collisions = np.append(collisions, all_collisions[i][:])
+                    watched_ribs.append(int(all_collisions.shape[1]))
+
+                else:
+
+                    collisions = np.append(collisions, all_collisions[i][:int(totalrib[0])])
+                    watched_ribs.append(int(totalrib[0]))
+
+            sttime = time.time() - st
+
+
+        no_ribosomes = np.zeros((n_traj, (len(k)+1)))
+
+        startindex = np.where(t >= non_consider_time)[0][0]
+
+        all_col_points2 = []
+        for i in range(len(all_col_points)):
+            all_col_points2.append(all_col_points[i][ all_col_points[i][:, 1] > non_consider_time])
+
+
+        rib_per_t = np.zeros((n_traj, len(t)))
+
+
+        validind = 0
+        riblocs = []
+        for i in range(len(all_rib_loc)):
+            try:
+
+
+                if np.where(np.sum(all_rib_loc[i].T, axis=1)!=0)[0][-1] > validind:
+
+                    validind = np.where(np.sum(all_rib_loc[i].T, axis=1)!=0)[0][-1]
+
+                        #there was a blank
+            except:
+                pass
+
+
+        all_rib_loc = all_rib_loc[:, :, :validind]
+
+        for i in range(all_rib_loc.shape[0]):
+            rib_loc = all_rib_loc[i, :, :]
+            rt = np.count_nonzero(rib_loc.T,axis=0)
+            rib_per_t[i, :] = rt
+
+        rib_density_loc, rib_density_count = np.unique(all_rib_loc, return_counts=True)
+        rib_density_loc = rib_density_loc[1:]
+        rib_density_count = rib_density_count[1:]
+        rib_density = rib_density_count / np.sum(rib_density_count)
+
+
+        ssa_obj = SSA_Soln()
+        ssa_obj.no_ribosomes = no_ribosomes
+        ssa_obj.n_traj = n_traj
+        ssa_obj.k = k
+        # ssa_obj.no_rib_per_mrna = no_ribosomes_per_mrna
+        ssa_obj.rib_per_t = rib_per_t.T[startindex:, :]
+        ssa_obj.rib_mean = np.mean(rib_per_t.T[startindex:, :])
+
+        ssa_obj.rib_density = rib_density
+        # ssa_obj.rib_means = ribosome_means
+        ssa_obj.intensity_vec = all_results.T[:, startindex:, :]
+
+        ssa_obj.time = t
+        ssa_obj.time_rec = t[startindex:]
+        ssa_obj.start_time = non_consider_time
+        ssa_obj.start_index = int(startindex)
+        ssa_obj.watched_ribs = watched_ribs
+        ssa_obj.collisions = collisions
+        ssa_obj.ribosome_locations = all_rib_loc
+        ssa_obj.ribtimes = all_ribtimes
+        ssa_obj.photobleaching = photobleaching
+        ssa_obj.photobleaching_rate = photobleaching_rate
+        ssa_obj.protease_locations = protease_locations
+        ssa_obj.protease_rates = protease_rates
+
+        try:
+            ssa_obj.col_points = all_col_points2
+        except:
+            pass
+
+
+        ssa_obj.eval_time = sttime
+
+        return ssa_obj
+
+
     def __solve_ssa_full_combined(self,k, t, x0, n_traj, ssa_conditions=None,
                                   kon=1, koff=1, kprobe=[] ):
         seeds = np.random.randint(0, 0x7FFFFFF, n_traj, dtype=np.int32)
